﻿Imports System.Windows.Resources
Imports System.Xml.Linq
Imports IdeaBlade.EntityModel

Namespace DomainModel
	''' <summary>
	''' Constructs Contact entity graphs from sample data in XML.
	''' The entities are held in an <see cref="EntityManager"/> to
	''' support property navigation among related entities
	''' </summary>
	Public NotInheritable Class ContactFactoryFromXml
		''' <summary>
		''' Constructs Contact entity graphs from sample data in XML.
		''' Uses a hidden, dummy <see cref="EntityManager"/> which
		''' is offline and can neither fetch nor save.
		''' </summary>
		Private Sub New()
		End Sub
		Public Shared Function GetContactsFromSampleData() As IEnumerable(Of Contact)
			Dim dummyManager = New EntityManager(False) 'disconnected
			AddHandler dummyManager.Fetching, Sub() Throw New InvalidOperationException("Can't fetch with dummy")
			AddHandler dummyManager.Saving, Sub() Throw New InvalidOperationException("Can't save with dummy")
			Return AttachContactsFromSampleDataToManager(dummyManager)
		End Function
		''' <summary>
		''' Constructs Contact entity graphs from sample data in XML and adds
		''' them to the supplied <see cref="EntityManager"/>.
		''' </summary>
		Public Shared Function AddContactsFromSampleDataToManager(ByVal manager As EntityManager) As IEnumerable(Of Contact)
			Return PopulateManagerFromSampleData(manager, True) ' as new entities
		End Function

		''' <summary>
		''' Constructs Contact entity graphs from sample data in XML and 
		''' attaches them to the supplied <see cref="EntityManager"/> 
		''' as if they were unmodified, pre-existing entities retrieved from the database.
		''' </summary>
		Public Shared Function AttachContactsFromSampleDataToManager(ByVal manager As EntityManager) As IEnumerable(Of Contact)
			Return PopulateManagerFromSampleData(manager, False) ' as pre-entities
		End Function

		''' <summary>
		''' Constructs Contact entity graphs from sample data in XML and
		''' adds them to the supplied <see cref="EntityManager"/> 
		''' as new entities, ready to be saved to the database.
		''' </summary>
		Private Shared Function PopulateManagerFromSampleData(ByVal manager As EntityManager, ByVal asNewEntities As Boolean) As IEnumerable(Of Contact)
			Dim resource As StreamResourceInfo = Nothing

			Try
				resource = System.Windows.Application.GetResourceStream(New Uri(SourceFileUri, UriKind.Relative))

				Dim doc As XDocument = XDocument.Load(resource.Stream)

				Dim contactsQuery As IEnumerable(Of Contact) = From x In doc.Descendants("Customer")
				                                               Select CreateContactFromXElement(manager, x, asNewEntities)

				Return contactsQuery.ToList()

			Catch ex As Exception
				Throw ex ' a place to break
			Finally
				If resource IsNot Nothing Then
					resource.Stream.Close()
				End If
			End Try
		End Function

		''' <summary>Location of the sample data XML.</summary>
		Public Const SourceFileUri As String = "Contacts_MVVM;component/SampleData/ContactDataSource.xml"

		Private Shared Function CreateContactFromXElement(ByVal manager As EntityManager, ByVal x As XElement, ByVal asNewContact As Boolean) As Contact
			Dim newContact = New Contact With {.ContactID = Guid.NewGuid(), .Id = x.Element("Id").Value.Trim(), .Name = x.Element("Name").Value.Trim(), .Email = x.Element("Email").Value.Trim(), .Phone = x.Element("Phone").Value.Trim(), .Fax = x.Element("Fax").Value.Trim(), .Cell = x.Element("Cell").Value.Trim(), .Blog = x.Element("Blog").Value.Trim(), .Twitter = x.Element("Twitter").Value.Trim(), .Website = x.Element("Website").Value.Trim()}

			Dim newAddress = New PostalAddress With {.AddressID = Guid.NewGuid(), .Address = x.Element("Address").Value.Trim(), .StateCodeName = x.Element("State").Value.Trim(), .ZipCode = x.Element("ZipCode").Value.Trim()}

			If asNewContact Then ' Contact and Address should be new entities
				manager.AddEntity(newContact)
				newContact.Address = newAddress ' pulls newAddress into manager
			Else ' Contact and Address should seem to be unmodified, pre-fetche entities
				newContact.AddressID = newAddress.AddressID ' associate by FK id
				manager.AttachEntity(newContact)
				manager.AttachEntity(newAddress)
			End If

			Return newContact
		End Function


	End Class
End Namespace
