﻿Imports System.Collections
Imports IdeaBlade.EntityModel

Namespace DomainModel
	''' <summary>
	''' Repository for creation, deletion, and persistence of Contacts.
	''' Exposes a singleton <see cref="Instance"/> 
	''' to access the application-wide repository.
	''' </summary>
	Partial Public Class ContactsRepository
		Implements IContactsRepository
		Private Shared _repository As IContactsRepository

		''' <summary>
		''' Get the <see cref="IntersoftSampleEntities"/> EntityManager
		''' used by this repository (INTERNAL / TEST USE ONLY)
		''' </summary>
		Private privateManager As IntersoftSampleEntities
		Friend Property Manager() As IntersoftSampleEntities
			Get
				Return privateManager
			End Get
			Private Set(ByVal value As IntersoftSampleEntities)
				privateManager = value
			End Set
		End Property

		''' <summary>
		''' Get singleton instance of the application's 
		''' <see cref="IContactsRepository"/>.
		''' </summary>
		Public Shared Property Instance() As IContactsRepository
			Get
				If _repository IsNot Nothing Then
					Return _repository
				Else
					_repository = CreateRepository()
					Return _repository
				End If
			End Get
			Set(ByVal value As IContactsRepository)
				_repository = value
			End Set
		End Property

		''' <summary>
		''' Create a new instance of the application's 
		''' <see cref="IContactsRepository"/>.
		''' </summary>
		Public Shared Function CreateRepository() As IContactsRepository
			Return New ContactsRepository(EntityManagerFactory.Create())
		End Function

		''' <summary>
		''' Repository constructor
		''' </summary>
		''' <param name="entityManager">
		''' The <see cref="IntersoftSampleEntities"/> EntityManager
		''' that the repository will
		''' use to access Contacts from permanent storage.
		''' </param>
		Public Sub New(ByVal entityManager As IntersoftSampleEntities)
			Manager = entityManager
		End Sub

		''' <remarks>
		''' Technically orderBy is not specified in contract but
		''' this implementation orders by Id to be nice.
		''' QueryStrategy is the <see cref="Manager"/>'s DefaultQueryStrategy;
		''' likely to be "Normal" = load first time, cached thereafter.
		''' </remarks>
		Public Sub GetContacts(ByVal onSuccess As Action(Of IEnumerable(Of Contact)), ByVal onFail As Action(Of Exception)) Implements IContactsRepository.GetContacts
      Manager.Contacts.Include(Function(c) c.Address).OrderBy(Function(c) c.Id).ExecuteAsync(Sub(op As EntityQueryOperation(Of Contact))
                                                                                               If op.CompletedSuccessfully Then
                                                                                                 If onSuccess IsNot Nothing Then
                                                                                                   onSuccess(op.Results)
                                                                                                 End If
                                                                                               Else
                                                                                                 If onFail IsNot Nothing Then
                                                                                                   op.MarkErrorAsHandled()
                                                                                                   onFail(op.Error)
                                                                                                 End If
                                                                                               End If
                                                                                             End Sub)
		End Sub

    Public Sub Save(ByVal onSuccess As Action(Of System.Collections.IEnumerable), ByVal onFail As Action(Of Exception)) Implements IContactsRepository.Save
      Manager.SaveChangesAsync(Sub(op)
                                 If op.CompletedSuccessfully Then
                                   If Nothing IsNot onSuccess Then
                                     onSuccess(op.Entities)
                                   End If
                                 Else
                                   If Nothing IsNot onFail Then
                                     op.MarkErrorAsHandled()
                                     onFail(op.Error)
                                   End If
                                 End If
                               End Sub)
    End Sub

		Public Sub Rollback() Implements IContactsRepository.Rollback
			If Manager IsNot Nothing Then
				Manager.RejectChanges()
			End If
		End Sub

		Public Function CreateContact() As Contact Implements IContactsRepository.CreateContact
			Dim contact = New Contact With {.ContactID = Guid.NewGuid(), .Address = CreateNewAddress()}

			Manager.AddEntity(contact)
			Return contact
		End Function

		Private Function CreateNewAddress() As PostalAddress
			Dim address = New PostalAddress With {.AddressID = Guid.NewGuid()}
			Manager.AddEntity(address)
			Return address
		End Function

		Public Sub DeleteContact(ByVal contact As Contact) Implements IContactsRepository.DeleteContact
			If contact Is Nothing Then
				Return
			End If

			contact.EntityAspect.Delete()
			contact.Address.EntityAspect.Delete()
		End Sub

	End Class
End Namespace
