﻿Imports IdeaBlade.EntityModel

Namespace DomainModel
	''' <summary>
	''' Creates a <see cref="IntersoftSampleEntities"/>
	''' to use in the current execution context.
	''' </summary>
	''' <remarks>
	''' Creates the manager by delegating to a function.
	''' That function returns a design-time manager
	''' by default to make view development with visual design tools easier;
	''' see <see cref="DesignEntityManagerFactory"/>.
	''' <para>
	''' For runtime execution, the creation function should be
	''' set to the production manager via <see cref="UseRealEntityManager"/>;
	''' do it early in the application launch cycle such as in App.xaml.cs.
	''' </para><para>
	''' The creation function also can be reset directly by components 
	''' in this assembly or by "friend" assemblies (e.g., test assemblies);
	''' see <see cref="SetCreateFn"/>.
	''' </para>
	''' </remarks>
	Public NotInheritable Class EntityManagerFactory
		''' <summary>
		''' Create an <see cref="IntersoftSampleEntities"/> 
		''' for the current execution context.
		''' </summary>
		Private Sub New()
		End Sub
    Public Shared Function Create() As IntersoftSampleEntities
      Return _createFn
    End Function

		''' <summary>
		''' Create a production <see cref="IntersoftSampleEntities"/> 
		''' upon next call to <see cref="Create"/>
		''' </summary>
    Public Shared Sub UseRealEntityManager()
      _createFn = New RealEntityManagerFactory().Create()
    End Sub

		''' <summary>
		''' Set the creation function that <see cref="Create"/> 
		''' uses to make a new <see cref="IntersoftSampleEntities"/>. 
		''' </summary>
    'Friend Shared Sub SetCreateFn(ByVal fn As Func(Of IntersoftSampleEntities))
    Friend Shared Sub SetCreateFn(ByVal fn As IntersoftSampleEntities)
      _createFn = fn
    End Sub

		''' <summary>
		''' Function that creates a new <see cref="IntersoftSampleEntities"/>.
		''' </summary>
    'Private Shared _createFn As Func(Of IntersoftSampleEntities) = New DesignEntityManagerFactory().Create
    Private Shared _createFn As IntersoftSampleEntities

	End Class

	''' <summary>
	''' Creates a production <see cref="IntersoftSampleEntities"/> 
	''' that accesses the database. 
	''' </summary>
	Friend Class RealEntityManagerFactory
		Private Shared _counter As Integer = 1 ' for debugging

		''' <summary>
		''' Creates a production <see cref="IntersoftSampleEntities"/> 
		''' that accesses the database. 
		''' </summary>
		Public Function Create() As IntersoftSampleEntities
			Dim manager = New IntersoftSampleEntities()
			manager.Tag = _counter
			_counter += 1
			AddHandler manager.Saving, AddressOf SaveValidationHelper.SavingHandler
			Return manager
		End Function

	End Class

	''' <summary>
	''' Creates an offline <see cref="IntersoftSampleEntities"/>
	''' prepared with design-time sample data.
	''' </summary>
	''' <remarks>
	''' Intended for use by ViewModels bound to Views
	''' during their development with visual design tools such as "Cider" and Blend. 
	''' The <see cref="EntityManager"/> is offline and can neither fetch nor save.
	''' </remarks>
	Friend Class DesignEntityManagerFactory
		''' <summary>
		''' Creates an offline <see cref="IntersoftSampleEntities"/>
		''' prepared with design-time sample data.
		''' </summary>
		Public Function Create() As IntersoftSampleEntities
			Dim manager = New IntersoftSampleEntities(False)
			AddHandler manager.Fetching, Sub() Throw New InvalidOperationException("Design manager can't fetch")
			AddHandler manager.Saving, Sub() Throw New InvalidOperationException("Design manager can't save")
			manager.DefaultQueryStrategy = QueryStrategy.CacheOnly

			ContactFactoryFromXml.AttachContactsFromSampleDataToManager(manager)

			Return manager

		End Function
	End Class
End Namespace