﻿Imports System.Text
Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel
Imports IdeaBlade.Validation

Namespace DomainModel
	''' <summary>
	''' Provides client-side event handlers for <see cref="EntityManager"/>s.
	''' </summary>
	Public Class SaveValidationHelper
		''' <summary>
		''' Saving handler that validates entities BEFORE they are saved.
		''' Also cancels the save if there is nothing to save.
		''' </summary>
		''' <exception cref="InvalidOperationException">
		''' Thrown with message describing all errant validation results
		''' if any entity to be saved fails validation.
		''' </exception>
		Public Shared Sub SavingHandler(ByVal sender As Object, ByVal e As EntitySavingEventArgs)
			If Not e.Entities.Any() Then
				e.Cancel = True ' nothing to validate
				Return
			End If
			SavingValidate(e, sender)
		End Sub

		' Validate everything we are about to save
		Private Shared Sub SavingValidate(ByVal e As EntitySavingEventArgs, ByVal sender As Object)
			FailOnValidationErrors(GetValidationErrors(TryCast(sender, EntityManager), e.Entities))
		End Sub

		Private Shared Function GetValidationErrors(ByVal manager As EntityManager, ByVal entities As IEnumerable(Of Object)) As Dictionary(Of Object, VerifierResultCollection)
			Dim validationErrors = New Dictionary(Of Object, VerifierResultCollection)()
			Dim verifierEngine = manager.VerifierEngine

      entities.ForEach(Sub(entity)
                         Dim selfValidator = TryCast(entity, ISelfValidator)
                         Dim valResults = If(selfValidator Is Nothing, verifierEngine.Execute(entity), selfValidator.Validate(verifierEngine, entity))
                         If Not valResults.HasErrors Then
                           Return
                         End If
                         validationErrors(entity) = valResults.Errors
                       End Sub)

			Return validationErrors
		End Function

		Private Shared Sub FailOnValidationErrors(ByVal validationErrors As Dictionary(Of Object, VerifierResultCollection))
			If validationErrors.Count = 0 Then
				Return
			End If

			Dim s = New StringBuilder("Entities to save failed validation:" & vbLf)
			validationErrors.ForEach(Function(ve) s.AppendLine(DescribeEntityValidationFailure(ve)))
			Throw New InvalidOperationException(s.ToString())
		End Sub

    Private Shared Function DescribeEntityValidationFailure(ByVal errs As KeyValuePair(Of Object, VerifierResultCollection)) As String
      Return errs.Key.ToString() & " - " & String.Join(", ", errs.Value.Select(Function(c) c.Message))
    End Function

  End Class
End Namespace