﻿Imports System.Collections
Imports System.ComponentModel
Imports DomainModel
Imports IdeaBlade.EntityModel
Imports Intersoft.Client.Framework
Imports Intersoft.Client.Framework.Input

Namespace Contacts_MVVM.ViewModels
	''' <summary>
	''' ViewModel that supports the contact editing view 
	''' <see cref="Contacts_MVVM.Views.ContactEditDialogBox"/>.
	''' </summary>
	''' <remarks>
	''' This ViewModel will save the changes when users clicked on the Save button.
	''' If the save failed, the changes are reversed (a new contact discarded)
	''' and then the dialog closes.
	''' </remarks>
	Public Class ContactEditViewModel
		Inherits ViewModelBase
		#Region "Fields"

		Private _contact As Contact
		Private _isNewContact As Boolean
		Private _isInvalidatingCommands As Boolean
		Private _isDirty As Boolean
		Private _requireValidate As Boolean
		Private _lastValidateResult As Boolean

		Private _isBusy As Boolean
		Private _contactEditDialogResult As DialogResult = DialogResult.None

		#End Region

		#Region "Commands"

		Public Property SaveCommand() As DelegateCommand

		#End Region

		#Region "Constructor"

		Public Sub New(ByVal contact As Contact, ByVal repository As IContactsRepository, ByVal closeEditView As Action)
			Me.Repository = repository
			Me.CloseEditView = closeEditView

			_contact = contact
			_isNewContact = contact.EntityAspect.EntityState = EntityState.Added

			If _isNewContact Then
				contact.Address.State = PostalAddress.DefaultStateCode
			End If

			AddHandler _contact.PropertyChanged, AddressOf Contact_PropertyChanged
			AddHandler _contact.Address.PropertyChanged, AddressOf Contact_PropertyChanged

			Me.SaveCommand = New DelegateCommand(AddressOf Save, AddressOf CanSaveContact)
			Me.MessagePresenter = New MessagePresenter()
		End Sub

		#End Region

		#Region "Properties"

		Public Property Repository() As IContactsRepository
		Public Property MessagePresenter() As IMessagePresenter
		Public Property CloseEditView() As Action

		Public ReadOnly Property DialogTitle() As String
			Get
				Return If(_isNewContact, "New Contact", "Edit Contact - " & _contact.Id & ", " & _contact.Name)
			End Get
		End Property

		Public ReadOnly Property Contact() As Contact
			Get
				Return _contact
			End Get
		End Property

		Public ReadOnly Property IsNewContact() As Boolean
			Get
				Return _isNewContact
			End Get
		End Property

		Public Property ContactEditDialogResult() As DialogResult
			Get
				Return _contactEditDialogResult
			End Get
			Private Set(ByVal value As DialogResult)
        If _contactEditDialogResult <> value Then
          _contactEditDialogResult = value
          OnPropertyChanged("ContactEditDialogResult")
        End If
			End Set
		End Property

		' Busy state
		Public Property IsBusy() As Boolean
			Get
				Return _isBusy
			End Get
			Friend Set(ByVal value As Boolean)
				If _isBusy <> value Then
					_isBusy = value
					OnPropertyChanged("IsBusy")
				End If
			End Set
		End Property

		#End Region

		#Region "Validation"

		Public Function Validate() As Boolean
			Return Me.Contact.Validate()
		End Function

		#End Region

		#Region "Save"

		Private Function CanSaveContact(ByVal parameter As Object) As Boolean
			' Can't save an untouched new contact
			If _isNewContact AndAlso (Not _isDirty) Then
				Return False
			End If

			If _requireValidate Then
				_lastValidateResult = Me.Validate()
				Return _lastValidateResult
			End If

			Return _lastValidateResult
		End Function

		Private Sub Save(ByVal parameter As Object)
			DetachEventHandlers()
      Me.IsBusy = False ' Busy indicator slows things down too much
			Repository.Save(AddressOf SaveSucceeded, AddressOf SaveFailed)
		End Sub

    Private Sub SaveSucceeded(ByVal entities As System.Collections.IEnumerable)
      Me.IsBusy = False
      DetachEventHandlers()
      Me.ContactEditDialogResult = DialogResult.OK
      CloseEditView()()
    End Sub

		Private Sub SaveFailed(ByVal [error] As Exception)
			Me.IsBusy = False

			Dim message As String = If([error] Is Nothing, "Save Cancelled!", "Error: " & [error].Message)

			DisplayMessage(message)
		End Sub

		Private Sub DisplayMessage(ByVal message As String)
			Me.MessagePresenter.ShowInformationMessage(message)
		End Sub

		#End Region

		#Region "Cancel"

		''' <summary>
		''' Cancel changes to the edited Contact
		''' </summary>
		Friend Sub Cancel()
			DetachEventHandlers()
		End Sub

		#End Region

		#Region "Invalidate Commands"

		Private Sub InvalidateCommands()
			' invalidate the commands that related to this view scope
			If _isInvalidatingCommands Then
				Return
			End If

			_isInvalidatingCommands = True
			Me.SaveCommand.RaiseCanExecuteChanged()
			_isInvalidatingCommands = False
		End Sub


		#End Region

		#Region "Event Handlers"

		Private Sub Contact_PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
			_isDirty = True
			_requireValidate = True
			InvalidateCommands()
			_requireValidate = False
		End Sub

		Private Sub DetachEventHandlers()
      RemoveHandler _contact.PropertyChanged, AddressOf Contact_PropertyChanged
      RemoveHandler _contact.Address.PropertyChanged, AddressOf Contact_PropertyChanged
		End Sub

		#End Region
	End Class
End Namespace
