﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports DomainModel
Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel
Imports Intersoft.Client.Framework
Imports Intersoft.Client.Framework.Input

Namespace Contacts_MVVM.ViewModels
	''' <summary>
	''' ViewModel that supports the Contacts view,
	''' <see cref="Contacts_MVVM.Views.ContactsView"/>.
	''' </summary>
	''' <remarks>
	''' See <see cref="SetDefaultContactEditor"/> to switch 
	''' between regular and sandboxed ContactEditor.
	''' </remarks>
	Public Class ContactsViewModel
		Inherits ViewModelBase
		#Region "Fields"

		Private _selectedItem As ContactViewModel
		Private _isInEditing As Boolean
		Private _isBusy As Boolean

		#End Region

		#Region "Commands"

		Private privateAddCommand As DelegateCommand
		Public Property AddCommand() As DelegateCommand
			Get
				Return privateAddCommand
			End Get
			Private Set(ByVal value As DelegateCommand)
				privateAddCommand = value
			End Set
		End Property
		Private privateEditCommand As DelegateCommand
		Public Property EditCommand() As DelegateCommand
			Get
				Return privateEditCommand
			End Get
			Private Set(ByVal value As DelegateCommand)
				privateEditCommand = value
			End Set
		End Property
		Private privateDeleteCommand As DelegateCommand
		Public Property DeleteCommand() As DelegateCommand
			Get
				Return privateDeleteCommand
			End Get
			Private Set(ByVal value As DelegateCommand)
				privateDeleteCommand = value
			End Set
		End Property
		Private privateResetDataCommand As DelegateCommand
		Public Property ResetDataCommand() As DelegateCommand
			Get
				Return privateResetDataCommand
			End Get
			Private Set(ByVal value As DelegateCommand)
				privateResetDataCommand = value
			End Set
		End Property

		#End Region

		#Region "Constructor"

		''' <summary>
		''' Constructs the ViewModel AND gets all contacts.
		''' </summary>
		Public Sub New()
			Me.AddCommand = New DelegateCommand(AddressOf AddContact, AddressOf CanAddContact)
			Me.EditCommand = New DelegateCommand(AddressOf EditContact, AddressOf CanEditContact)
			Me.DeleteCommand = New DelegateCommand(AddressOf DeleteContact, AddressOf CanDeleteContact)
			Me.ResetDataCommand = New DelegateCommand(AddressOf ResetData, AddressOf CanResetData)

			Me.Contacts = New ObservableCollection(Of ContactViewModel)()
			AddHandler Me.Contacts.CollectionChanged, AddressOf Contacts_CollectionChanged

			Me.Repository = ContactsRepository.Instance
			Me.MessagePresenter = New MessagePresenter()
			Me.DataSourceResetter = New DataSourceResetter()

			SetDefaultContactEditor()
			GetContacts()
		End Sub

		Private Sub SetDefaultContactEditor()
			Me.ContactEditor = New ContactEditor(Repository)
		End Sub

		#End Region

		#Region "Properties for Views, Selection and States"

		Public Property ContactEditor() As IContactEditor
		Public Property Repository() As IContactsRepository
		Public Property MessagePresenter() As IMessagePresenter
		Public Property DataSourceResetter() As IDataSourceResetter

		' Views
		Public Property Contacts() As ObservableCollection(Of ContactViewModel)

		Public ReadOnly Property ContactsCount() As String
			Get
				If Me.Contacts.Count = 0 Then
					Return "No contacts"
				End If
				If Me.Contacts.Count = 1 Then
					Return "1 contact"
				End If
				Return Me.Contacts.Count & " contacts"
			End Get
		End Property

		''' <summary>
		''' Get and set the currently selected
		''' ContactViewModel that wraps the current Contact.
		''' </summary>
		Public Property SelectedItem() As ContactViewModel
			Get
				Return _selectedItem
			End Get
			Set(ByVal value As ContactViewModel)
				If _selectedItem IsNot value Then
					If _selectedItem IsNot Nothing Then
            RemoveHandler _selectedItem.Contact.PropertyChanged, AddressOf Contact_PropertyChanged
					End If

					If value IsNot Nothing Then
						AddHandler value.Contact.PropertyChanged, AddressOf Contact_PropertyChanged
					End If

					_selectedItem = value
					OnPropertyChanged("SelectedItem")
					OnPropertyChanged("SelectionStatus")
					InvalidateCommands()
				End If
			End Set
		End Property

		Public Property IsInEditing() As Boolean
			Get
				Return _isInEditing
			End Get
			Private Set(ByVal value As Boolean)
				If _isInEditing <> value Then
					_isInEditing = value
					InvalidateCommands()
					OnPropertyChanged("IsInEditing")
				End If
			End Set
		End Property

		Public ReadOnly Property SelectionStatus() As String
			Get
				Return If(Me.SelectedItem Is Nothing, "No contact selected", "Selected contact: " & Me.SelectedItem.Contact.Name)
			End Get
		End Property

		Public Property IsBusy() As Boolean
			Get
				Return _isBusy
			End Get
			Friend Set(ByVal value As Boolean)
				If _isBusy <> value Then
					_isBusy = value
					OnPropertyChanged("IsBusy")
				End If
			End Set
		End Property

		#End Region

		#Region "Get Contacts"

		Private Sub GetContacts()
      Me.IsBusy = False ' Busy indicator slows things down too much

'INSTANT VB TODO TASK: The following line could not be converted:
      Repository.GetContacts(Sub(contacts)
                               Me.IsBusy = False
                               ResetContactsList(contacts)
                             End Sub, Sub(err)
                                        Me.IsBusy = False
                                        DisplayMessage(err.Message)
                                      End Sub)
		End Sub

    Public Sub ResetContactsList(ByVal contacts As IEnumerable(Of Contact))
      Me.Contacts.Clear()
      contacts.ForEach(Sub(c) Me.Contacts.Add(New ContactViewModel(c)))
    End Sub

		Private Sub DisplayMessage(ByVal message As String)
			Me.MessagePresenter.ShowInformationMessage(message)
		End Sub

		#End Region

		#Region "Add Contact"

		Private Function CanAddContact(ByVal parameter As Object) As Boolean
			Return Not IsInEditing
		End Function

		Private Sub AddContact(ByVal parameter As Object)
			Me.IsInEditing = True

			ContactEditor.AddNewContact(AddressOf AddedContactSaved, AddressOf EditorClosed) ' Cancelled callback -  Saved callback
		End Sub

		Private Sub AddedContactSaved(ByVal contact As Contact)
			Dim newContactView = New ContactViewModel(contact)
			Me.Contacts.Add(newContactView) ' Add new contact to list
			Me.SelectedItem = newContactView ' Position on the new contact
			EditorClosed()
		End Sub

		#End Region

		#Region "Edit Contact"

		Private Function CanEditContact(ByVal parameter As Object) As Boolean
			Return (Not IsInEditing) AndAlso SelectedItem IsNot Nothing
		End Function

		Private Sub EditContact(ByVal parameter As Object)
			Me.IsInEditing = True

      'ContactEditor.EditExistingContact(Me.SelectedItem.Contact, Sub() AddressOf EditorClosed, Sub() AddressOf EditorClosed)
      ContactEditor.EditExistingContact(Me.SelectedItem.Contact, Sub() EditorClosed(), Sub() EditorClosed()) ' Cancelled callback -  Saved callback -  Contact to edit

End Sub

		Private Sub EditorClosed()
			Me.IsInEditing = False
		End Sub

		#End Region

		#Region "Delete Contact"

		Private Function CanDeleteContact(ByVal parameter As Object) As Boolean
			Return (Not IsInEditing) AndAlso SelectedItem IsNot Nothing
		End Function

		Private Sub DeleteContact(ByVal parameter As Object)
      MessagePresenter.AskYesOrNo("Are you sure you want to delete the selected contact?", AddressOf DeleteContact, Nothing)
		End Sub

		Private Sub DeleteContact()
			Dim currentSelection = Me.SelectedItem
			Repository.DeleteContact(currentSelection.Contact)

			' Marks page as busy during delete
      Me.IsBusy = False ' Busy indicator slows things down too much

				' Save succeed
				' Save failed
      ' Displays the message box in the next thread
      ' Displays the message box in the next thread
     
      'Repository.Save(Sub(_) OnEntityDeleteCompleted(), Function(err) { Utility.ExecuteTimeOut(0.4, Function() { DisplayMessage("Error occurred: " & err.Message); OnEntityDeleteCompleted(); }); }) DEK Corrected Lambda
      Repository.Save(Sub(c) OnEntityDeleteCompleted(), _
        Sub(err) Utility.ExecuteTimeOut(0.4, Sub() _
          DisplayMessage("Error occurred: " & err.Message)))

    End Sub

    Private Sub OnEntityDeleteCompleted()
      Dim selected = Me.SelectedItem
      If selected Is Nothing Then
        Return
      End If

      Dim contact = selected.Contact
      Dim entityState = contact.EntityAspect.EntityState

      Me.IsBusy = False

      If entityState = entityState.Deleted Then
        ' Still marked 'Deleted'? 
        ' Save must have failed; 'undo' delete
        contact.EntityAspect.RejectChanges()
      End If

      If entityState = entityState.Detached Then
        ' After successful delete or 'undo' of added contact
        Me.SelectedItem = Nothing
        Me.Contacts.Remove(selected)
      End If
    End Sub

#End Region

#Region "Reset Data"

    Private Function CanResetData(ByVal parameter As Object) As Boolean
      Return Not IsInEditing
    End Function

    Private Sub ResetData(ByVal parameter As Object)
      Me.IsBusy = False ' Busy indicator slows things down too much

      DataSourceResetter.ResetContacts(Repository, Sub(contacts)
                                                     Me.IsBusy = False
                                                     ResetContactsList(contacts)
                                                     DisplayMessage("Datasource reset from sample data")
                                                   End Sub, Sub(err)
                                                              Me.IsBusy = False
                                                              DisplayMessage(err.Message)
                                                            End Sub)
    End Sub

#End Region

#Region "Invalidate Commands"

    Private Sub InvalidateCommands()
      ' Invalidate the commands that related to this view scope

      Me.AddCommand.RaiseCanExecuteChanged()
      Me.EditCommand.RaiseCanExecuteChanged()
      Me.DeleteCommand.RaiseCanExecuteChanged()
      Me.ResetDataCommand.RaiseCanExecuteChanged()
    End Sub

#End Region

#Region "Event Handlers"

    Private Sub Contact_PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
      If e.PropertyName = "Name" OrElse String.IsNullOrEmpty(e.PropertyName) Then
        OnPropertyChanged("SelectionStatus")
      End If
    End Sub

    Private Sub Contacts_CollectionChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
      OnPropertyChanged("ContactsCount")
    End Sub

#End Region
  End Class
End Namespace
